@@ -298,4 +298,153 @@ mod test {
     assert_eq!(q, q_r);
   }
 
+  #[test]
+  fn complex_optional_1() {
+    let entity: Option<i32> = None;
+    let state = Some("Pending");
+    let currency_iso = Some("USD");
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::And {
+        left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+        right: Box::new(Query::Eq { field: "state".to_owned(), value: "Pending".into() }),
+      }),
+      right: Box::new(Query::Or {
+        left: Box::new(Query::Eq { field: "source.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+        right: Box::new(Query::Eq { field: "destination.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+      })
+    };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_2() {
+    let entity = Some(12);
+    let state: Option<String> = None;
+    let currency_iso = Some("USD");
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::And {
+        left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+        right: Box::new(Query::Or {
+          left: Box::new(Query::Eq { field: "source.id".to_owned(), value: 12.into() }),
+          right: Box::new(Query::Eq { field: "destination.id".to_owned(), value: 12.into() }),
+        })
+      }),
+      right: Box::new(Query::Or {
+        left: Box::new(Query::Eq { field: "source.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+        right: Box::new(Query::Eq { field: "destination.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+      })
+    };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_3() {
+    let entity = Some(12);
+    let state = Some("Pending");
+    let currency_iso: Option<String> = None;
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::And {
+        left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+        right: Box::new(Query::Or {
+          left: Box::new(Query::Eq { field: "source.id".to_owned(), value: 12.into() }),
+          right: Box::new(Query::Eq { field: "destination.id".to_owned(), value: 12.into() }),
+        })
+      }),
+      right: Box::new(Query::Eq { field: "state".to_owned(), value: "Pending".into() }),
+    };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_4() {
+    let entity: Option<i32> = None;
+    let state = Some("Pending");
+    let currency_iso: Option<String> = None;
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+      right: Box::new(Query::Eq { field: "state".to_owned(), value: "Pending".into() }),
+    };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_5() {
+    let entity: Option<i32> = None;
+    let state: Option<String> = None;
+    let currency_iso: Option<String> = None;
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::Eq { field: "deleted".to_owned(), value: false.into() };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_6() {
+    let entity: Option<i32> = Some(12);
+    let state: Option<String> = None;
+    let currency_iso: Option<String> = None;
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+      right: Box::new(Query::Or {
+        left: Box::new(Query::Eq { field: "source.id".to_owned(), value: 12.into() }),
+        right: Box::new(Query::Eq { field: "destination.id".to_owned(), value: 12.into() }),
+      })
+    };
+    assert_eq!(q, q_r);
+  }
+
+  #[test]
+  fn complex_optional_7() {
+    let entity: Option<i32> = None;
+    let state: Option<String> = None;
+    let currency_iso: Option<&str> = Some("USD");
+    let q = query!(
+      (
+        ("deleted" == false && cond(entity.is_some()) ("source.id" == { entity.clone().unwrap() } || "destination.id" == { entity.clone().unwrap() } )) &&
+        cond(state.is_some()) "state" == { state.unwrap() }
+      ) && cond(currency_iso.is_some()) ("source.currency_iso" == { currency_iso.clone().unwrap() } || ("destination.currency_iso" == { currency_iso.clone().unwrap() }))
+    );
+    let q_r = Query::And {
+      left: Box::new(Query::Eq { field: "deleted".to_owned(), value: false.into() }),
+      right: Box::new(Query::Or {
+        left: Box::new(Query::Eq { field: "source.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+        right: Box::new(Query::Eq { field: "destination.currency_iso".to_owned(), value: "USD".to_owned().into() }),
+      })
+    };
+    assert_eq!(q, q_r);
+  }
 }
\ No newline at end of file
